@@ -9,6 +9,8 @@
  */
 
 #include "grib_tools.h"
+#include <stdlib.h>
+
 #if HAVE_LIBJASPER
 /* Remove compiler warnings re macros being redefined */
 #undef PACKAGE_BUGREPORT
@@ -295,6 +297,7 @@ static int grib_tool_with_orderby(grib_runtime_options* options)
         grib_handle_delete(h);
     }
 
+    if (set->size==0) fprintf(stderr, "no messages found in fieldset\n");
     grib_tool_finalise_action(options);
 
     grib_fieldset_delete(set);
@@ -416,7 +419,7 @@ static int grib_tool_without_orderby(grib_runtime_options* options)
             fclose(infile->file);
 
         if (infile->handle_count == 0) {
-            fprintf(dump_file, "no messages found in %s\n", infile->name);
+            fprintf(stderr, "no messages found in %s\n", infile->name);
             if (options->fail)
                 exit(1);
         }
@@ -733,7 +736,7 @@ static void grib_tools_set_print_keys(grib_runtime_options* options, grib_handle
     if (ns) {
         kiter = grib_keys_iterator_new(h, 0, ns);
         if (!kiter) {
-            fprintf(dump_file, "ERROR: Unable to create keys iterator\n");
+            fprintf(stderr, "ERROR: Unable to create keys iterator\n");
             exit(1);
         }
 
@@ -829,6 +832,17 @@ void grib_skip_check(grib_runtime_options* options, grib_handle* h)
 {
     int i, ret = 0;
     grib_values* v = NULL;
+
+    /* ECC-1179: bufr_copy/bufr_ls: Allow 'where' clause with Data Section keys */
+    if (options->constraints_count > 0 && h->product_kind == PRODUCT_BUFR) {
+        for (i = 0; i < options->set_values_count; i++) {
+            if (strcmp(options->set_values[i].name, "unpack")==0) {
+                grib_set_long(h, "unpack", 1);
+                break;
+            }
+        }
+    }
+
     for (i = 0; i < options->constraints_count; i++) {
         v = &(options->constraints[i]);
         if (v->equal) {
@@ -903,7 +917,8 @@ static int is_valid_JSON_number(const char* input)
     return 1;
 }
 
-static void get_value_for_key(grib_handle* h, const char* key_name, int key_type, char* value_str, const char* format)
+static void get_value_for_key(grib_handle* h, const char* key_name, int key_type, char* value_str,
+                              const char* format, int fix_lsdate, int fix_lstime)
 {
     int ret = 0, type = key_type;
     double dvalue = 0;
@@ -928,7 +943,11 @@ static void get_value_for_key(grib_handle* h, const char* key_name, int key_type
     }
 
     if (type == GRIB_TYPE_STRING) {
-        ret = grib_get_string(h, key_name, value_str, &len);
+        const char* pName = key_name;
+        /* ECC-707 */
+        if (fix_lsdate && strcmp(pName, "date") == 0) pName = "ls.date";
+        if (fix_lstime && strcmp(pName, "time") == 0) pName = "ls.time";
+        ret = grib_get_string(h, pName, value_str, &len);
     }
     else if (type == GRIB_TYPE_DOUBLE) {
         ret = grib_get_double(h, key_name, &dvalue);
@@ -961,9 +980,14 @@ static int fix_for_lsdate_needed(grib_handle* h)
 {
     long lsdate_bug = 0;
     int err         = grib_get_long(h, "lsdate_bug", &lsdate_bug);
-    if (!err && lsdate_bug == 1) {
-        return 1;
-    }
+    if (!err && lsdate_bug == 1) return 1;
+    return 0;
+}
+static int fix_for_lstime_needed(grib_handle* h)
+{
+    long lstime_bug = 0;
+    int err         = grib_get_long(h, "lstime_bug", &lstime_bug);
+    if (!err && lstime_bug == 1) return 1;
     return 0;
 }
 
@@ -1039,16 +1063,21 @@ void grib_print_key_values(grib_runtime_options* options, grib_handle* h)
     grib_accessor* acc   = NULL;
     size_t num_vals      = 0;
     int fix_lsdate       = 0;
+    int fix_lstime       = 0;
 
     if (!options->verbose)
         return;
 
+    fix_lsdate = (options->name_space && strcmp(options->name_space, "ls") == 0 && fix_for_lsdate_needed(h));
+    fix_lstime = (options->name_space && strcmp(options->name_space, "ls") == 0 && fix_for_lstime_needed(h));
+
     if (options->json_output && !options->latlon) {
         /* fprintf(dump_file, "\"message %d\" : {\n", options->handle_count); */
         fprintf(dump_file, "  {\n");
         for (i = 0; i < options->print_keys_count; i++) {
             fprintf(dump_file, "    \"%s\": ", options->print_keys[i].name);
-            get_value_for_key(h, options->print_keys[i].name, options->print_keys[i].type, value, options->format);
+            get_value_for_key(h, options->print_keys[i].name, options->print_keys[i].type, value,
+                              options->format, fix_lsdate, fix_lstime);
             if (is_valid_JSON_number(value))
                 fprintf(dump_file, "%s", value);
             else
@@ -1062,8 +1091,6 @@ void grib_print_key_values(grib_runtime_options* options, grib_handle* h)
         return;
     }
 
-    fix_lsdate = (fix_for_lsdate_needed(h) && options->name_space && strcmp(options->name_space, "ls") == 0);
-
     for (i = 0; i < options->print_keys_count; i++) {
         size_t len = MAX_STRING_LEN;
         ret        = GRIB_SUCCESS;
@@ -1127,6 +1154,9 @@ void grib_print_key_values(grib_runtime_options* options, grib_handle* h)
                         if (fix_lsdate && strcmp(pName, "date") == 0) { /* ECC-707 */
                             pName = "ls.date";
                         }
+                        if (fix_lstime && strcmp(pName, "time") == 0) {
+                            pName = "ls.time";
+                        }
                         ret = grib_get_size(h, pName, &num_vals);
                         if (ret == GRIB_SUCCESS && num_vals > 1) { /* See ECC-278 */
                             ret = get_initial_element_of_array(h, pName, num_vals, value);
@@ -1275,6 +1305,18 @@ void grib_print_full_statistics(grib_runtime_options* options)
                 options->filter_handle_count, options->handle_count, options->file_count);
 }
 
+static int filenames_equal(const char* f1, const char* f2)
+{
+    int eq = 0;
+    grib_context* c = grib_context_get_default();
+    char* resolved1 = codes_resolve_path(c, f1);
+    char* resolved2 = codes_resolve_path(c, f2);
+    eq = (strcmp(resolved1, resolved2)==0);
+    grib_context_free(c, resolved1);
+    grib_context_free(c, resolved2);
+    return eq;
+}
+
 void grib_tools_write_message(grib_runtime_options* options, grib_handle* h)
 {
     const void* buffer;
@@ -1295,6 +1337,13 @@ void grib_tools_write_message(grib_runtime_options* options, grib_handle* h)
 
     err = grib_recompose_name(h, NULL, options->outfile->name, filename, 0);
 
+    /* Check outfile is not same as infile */
+    if (filenames_equal(options->infile->name, filename)) {
+        grib_context_log(h->context, GRIB_LOG_ERROR,
+                "output file '%s' is the same as input file. Aborting\n", filename);
+        exit(GRIB_IO_PROBLEM);
+    }
+
     of = grib_file_open(filename, "w", &err);
 
     if (!of || !of->handle) {
